'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const erc780_1 = require("./erc780");
const ethUtil = require("ethereumjs-util");
const HDKey = require("hdkey");
const bitcore_lib_1 = require("bitcore-lib");
class Erc780HdKeyHash2018 {
    constructor(ethereumProvider, contractAddress, privateKey, options) {
        this.attestPrivate = (pubExtKey, predicate, object) => {
            const subject = this.pubKeyFromPubExtKey(pubExtKey, predicate);
            return this._erc780.setClaim(this._hash(subject), this._hash(subject + predicate), this._hash(subject + object));
        };
        this.attestPublic = (predicate, object) => {
            const subject = this._erc780.account.publicKey;
            return this._erc780.setClaim(this._hash(subject), this._hash(subject + predicate), this._hash(subject + object));
        };
        this.revokePrivate = (pubExtKey, predicate) => {
            const subject = this.pubKeyFromPubExtKey(pubExtKey, predicate);
            return this._erc780.removeClaim(this._erc780.account.address, this._hash(subject), this._hash(subject + predicate));
        };
        this.revokePublic = (predicate) => {
            const subject = this._erc780.account.publicKey;
            return this._erc780.removeClaim(this._erc780.account.address, this._hash(subject), this._hash(subject + predicate));
        };
        this.verify = (attestor, subject, predicate, object) => {
            return this._erc780.getClaim(attestor, this._hash(subject), this._hash(subject + predicate)).then(value => {
                return value.slice(0, 42) === this._hash(subject + object);
            });
        };
        this.ecdsaKoblitzPubKeyFromPubKey = (pubKey) => {
            return bitcore_lib_1.PublicKey(pubKey.replace(/^0x/, '')).toAddress().toString();
        };
        this.pubKeyFromPubExtKey = (pubExtKey, predicate) => {
            let hdkey;
            if (predicate) {
                const shortIri = predicate.split('/').pop();
                const hdkeyPath = 'm/' + this._get32BitHash(shortIri);
                hdkey = HDKey.fromExtendedKey(pubExtKey).derive(hdkeyPath);
            }
            else {
                hdkey = HDKey.fromExtendedKey(pubExtKey);
            }
            return '0x' + hdkey.publicKey.toString('hex');
        };
        // Create 32bit hash predicateId so it can be used in HDKey
        // This way we no longer have to map predicates to integers
        this._get32BitHash = (buf) => {
            // return ethUtil.sha256(Buffer.from(buf, 'utf8')).readUInt32BE(0) >>> 1;
            return Math.floor(ethUtil.sha256(Buffer.from(buf, 'utf8')).readUInt32BE(0) / 2);
        };
        this._hash = (data) => {
            return ('0x' + ethUtil.sha3(data).slice(-20).toString('hex'));
        };
        this._erc780 = new erc780_1.Erc780(ethereumProvider, contractAddress, privateKey, options);
    }
    get address() {
        return this._erc780.account.address;
    }
}
exports.Erc780HdKeyHash2018 = Erc780HdKeyHash2018;
exports.default = Erc780HdKeyHash2018;
//# sourceMappingURL=erc780-hdkey-hash-2018.js.map