'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Web3 = require("web3");
const EthTx = require("ethereumjs-tx");
class Erc780Options {
}
exports.Erc780Options = Erc780Options;
class Erc780 {
    constructor(ethereumProvider, contractAddress, privateKey, options = {}) {
        this._pickDefault = function (obj1, obj2) {
            if (typeof (obj1) === 'undefined') {
                return obj2;
            }
            else {
                return obj1;
            }
        };
        this._connectWeb3 = function () {
            return __awaiter(this, void 0, void 0, function* () {
                // Try to catch a lost connection -- not sure if this works as intended
                if (!this._web3) {
                    this._web3 = new Web3(Web3.givenProvider || this._ethereumProvider);
                    Promise.resolve();
                }
                yield this._web3.eth.net.isListening().catch(() => {
                    this._web3 = new Web3(Web3.givenProvider || this._ethereumProvider);
                    Promise.resolve();
                });
            });
        };
        this.setClaim = (subject, key, value) => {
            return this._sendSignedTransaction('setClaim', [subject, key, value]);
        };
        this.setSelfClaim = (key, value) => {
            return this._sendSignedTransaction('setSelfClaim', [key, value]);
        };
        this.removeClaim = (issuer, subject, key) => {
            return this._sendSignedTransaction('removeClaim', [issuer, subject, key]);
        };
        this.getClaim = (issuer, subject, key) => {
            return this._contract.methods.getClaim(issuer, subject, key).call();
        };
        this._sendSignedTransaction = (method, parameters) => __awaiter(this, void 0, void 0, function* () {
            if (!this._account) {
                return Promise.reject(`Error: Can not call "${method}" without privateKey`);
            }
            this._connectWeb3();
            const methodFn = this._contract.methods[method](...parameters);
            const data = methodFn.encodeABI();
            const [gas, nonce] = yield Promise.all([
                Promise.resolve(300000),
                this._transactionCount.transactionCount()
            ]);
            const rawTx = {
                nonce: this._web3.utils.toHex(nonce),
                gasPrice: this._web3.utils.toHex(Math.min(gas * this._gasMultiplier, this._gasPriceMax)),
                gasLimit: this._web3.utils.toHex(this._gasLimit),
                to: this._contractAddress,
                value: '0x00',
                data
            };
            const ethTx = new EthTx(rawTx);
            ethTx.sign(new Buffer(this._account.privateKey.slice(2), 'hex'));
            const serializedTx = ethTx.serialize();
            return new Promise((resolve, reject) => {
                this._web3.eth.sendSignedTransaction('0x' + serializedTx.toString('hex'))
                    .once('transactionHash', txHash => {
                    resolve(txHash);
                })
                    .once('error', error => {
                    reject(error);
                });
            });
        });
        this._ethereumProvider = ethereumProvider;
        this._gasMultiplier = this._pickDefault(options.gasMultiplier, DEFAULT_GAS_MULTIPLIER);
        this._gasLimit = this._pickDefault(options.gasLimit, DEFAULT_GAS_LIMIT);
        this._gasPriceMax = this._pickDefault(options.gasPriceMax, DEFAULT_GAS_PRICE_MAX);
        this._contractAddress = contractAddress;
        this._ABI = this._pickDefault(options.ABI, DEFAULT_ABI);
        this._connectWeb3();
        this._contract = new this._web3.eth.Contract(this._ABI, this._contractAddress);
        if (privateKey) {
            this._account = this._web3.eth.accounts.privateKeyToAccount('0x' + privateKey);
            this._transactionCount = new TransactionCount(this._web3, this._account.address);
        }
    }
    get ethereumProvider() {
        return this._ethereumProvider;
    }
    get contractAddress() {
        return this._contractAddress;
    }
    get web3() {
        return this._web3;
    }
    get account() {
        return this._account;
    }
}
exports.Erc780 = Erc780;
class TransactionCount {
    constructor(web3, address) {
        /*********************************************************************************
         * currentTransaction holds the last transactionCount
         * Race conditions might occur. try to manage:
         * - transactionNr missed. reset if raceCount > maxRaceCount
         * - idle for some time. reset if lastTxTime > maxIdleTime
         * In the future you might want to handle this completely different by keeping
         * track of all open requests
         ********************************************************************************/
        this._currentTransaction = 0;
        this._raceCount = 0;
        this._lastTxTime = 0;
        this.transactionCount = () => __awaiter(this, void 0, void 0, function* () {
            const maxRaceCount = 100;
            const maxIdleTime = 30 * 1000; // Make sure to skip at least 1 block
            const now = new Date().valueOf(); // Time in miliseconds since 1970
            const nonce = yield this._web3.eth.getTransactionCount(this._address);
            if ((nonce > this._currentTransaction) ||
                (this._raceCount > maxRaceCount) ||
                ((now - this._lastTxTime) > maxIdleTime)) {
                // Normal situation
                this._raceCount = 0;
                this._lastTxTime = now;
                this._currentTransaction = nonce;
                return this._currentTransaction;
            }
            else {
                // We might be racing. Add one more transaction
                this._raceCount++;
                this._lastTxTime = now;
                this._currentTransaction++;
                return this._currentTransaction;
            }
        });
        this._web3 = web3;
        this._address = address;
    }
}
const DEFAULT_GAS_MULTIPLIER = 10000;
const DEFAULT_GAS_PRICE_MAX = 100000000000;
const DEFAULT_GAS_LIMIT = 300000;
const DEFAULT_ABI = [{
        'constant': false,
        'inputs': [{
                'name': 'key',
                'type': 'bytes32'
            },
            {
                'name': 'value',
                'type': 'bytes32'
            }
        ],
        'name': 'setSelfClaim',
        'outputs': [],
        'payable': false,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': false,
        'inputs': [{
                'name': 'subject',
                'type': 'address'
            },
            {
                'name': 'key',
                'type': 'bytes32'
            },
            {
                'name': 'value',
                'type': 'bytes32'
            }
        ],
        'name': 'setClaim',
        'outputs': [],
        'payable': false,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': false,
        'inputs': [{
                'name': 'issuer',
                'type': 'address'
            },
            {
                'name': 'subject',
                'type': 'address'
            },
            {
                'name': 'key',
                'type': 'bytes32'
            }
        ],
        'name': 'removeClaim',
        'outputs': [],
        'payable': false,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': true,
        'inputs': [{
                'name': 'issuer',
                'type': 'address'
            },
            {
                'name': 'subject',
                'type': 'address'
            },
            {
                'name': 'key',
                'type': 'bytes32'
            }
        ],
        'name': 'getClaim',
        'outputs': [{
                'name': '',
                'type': 'bytes32'
            }],
        'payable': false,
        'stateMutability': 'view',
        'type': 'function'
    },
    {
        'constant': true,
        'inputs': [{
                'name': '',
                'type': 'address'
            },
            {
                'name': '',
                'type': 'address'
            },
            {
                'name': '',
                'type': 'bytes32'
            }
        ],
        'name': 'registry',
        'outputs': [{
                'name': '',
                'type': 'bytes32'
            }],
        'payable': false,
        'stateMutability': 'view',
        'type': 'function'
    },
    {
        'anonymous': false,
        'inputs': [{
                'indexed': true,
                'name': 'issuer',
                'type': 'address'
            },
            {
                'indexed': true,
                'name': 'subject',
                'type': 'address'
            },
            {
                'indexed': true,
                'name': 'key',
                'type': 'bytes32'
            },
            {
                'indexed': false,
                'name': 'value',
                'type': 'bytes32'
            },
            {
                'indexed': false,
                'name': 'updatedAt',
                'type': 'uint256'
            }
        ],
        'name': 'ClaimSet',
        'type': 'event'
    },
    {
        'anonymous': false,
        'inputs': [{
                'indexed': true,
                'name': 'issuer',
                'type': 'address'
            },
            {
                'indexed': true,
                'name': 'subject',
                'type': 'address'
            },
            {
                'indexed': true,
                'name': 'key',
                'type': 'bytes32'
            },
            {
                'indexed': false,
                'name': 'removedAt',
                'type': 'uint256'
            }
        ],
        'name': 'ClaimRemoved',
        'type': 'event'
    }
];
exports.default = Erc780;
//# sourceMappingURL=erc780.js.map