import * as enumerations from './enumerations';
/**
 * Interfaces between Frontend and Backend
 */
export interface ISsifBackend {
    ssifRequest(request: ISsifRequest): Boolean;
    ssifCreateSessionId(): String;
    callbackSession(sessionId: String): ISsifResult;
    getSessionRequest(sessionId: String): ISsifSessionRequest;
    postSessionResponse(response: ISsifSessionResponse): Boolean;
}
export interface ISsifApi {
    createSessionRequestAnonymous(request: ISsifRequest): ISsifSessionRequest;
    createSessionRequest(request: ISsifCreateRequest): ISsifSessionRequest;
    executeSession(create: ISsifCreateRequest, request: ISsifSessionRequest, response: ISsifSessionResponse): Promise<ISsifResult>;
}
/**
 *  Interfaces between Backend and SsifApi
 */
export interface ISsifCommitment {
    [key: string]: any;
}
export interface ISsifStatements {
    [predicate: string]: any;
}
export interface ISsifSpo {
    '@context': string[];
    '@type': string;
    statements: ISsifStatements;
    commitment?: ISsifCommitment;
}
export interface ISsifTrustedAttestor {
    type: enumerations.SsifTrustIssuerType;
    name: string;
}
export interface ISsifAttest {
    iri: string;
    value: string;
}
export interface ISsifText {
    langcode: string;
    text: string;
}
export interface ISsifVerifyRequest {
    iri: string;
    value: string;
    trustedAttestor: ISsifTrustedAttestor[];
}
export interface ISsifAttestation {
    name: string;
    spo: ISsifSpo;
}
export interface ISsifAttestor {
    name: string;
    ethereumAdress: string;
    logo: string;
    infoUrl: string;
    pssifUrl: string;
    texts: ISsifText[];
}
export interface ISsifCategory {
    name: string;
    texts: ISsifText[];
}
export interface ISsifMeta {
    attestationIssuer: ISsifSpo;
}
export interface ISsifPredicate {
    iri: string;
    datatype: string;
    texts: ISsifText[];
    categoryName: string;
    attestorNames: string[];
}
export interface ISsifRequest {
    attest?: Record<string, string>;
    revoke?: string[];
    verifyRequest?: string[];
}
export interface ISsifAttestorInformation {
    name: string;
    ethereumAdress: string;
    logo: string;
    infoUrl: string;
    ssifUrl: string;
    texts: ISsifText[];
    responsePredicates: ISsifPredicate[];
}
export interface ISsifConfiguration {
    blockchain?: string;
    privateKey?: string;
    categories?: ISsifCategory[];
    sessionTimeout?: number;
    gasMultiplier?: number;
    gasLimit?: number;
    gasPriceMax?: number;
}
export interface ISsifCreateRequest extends ISsifRequest {
    publicExtendedKey: string;
}
export interface ISsifResult {
    verified?: ISsifSpo[];
    revoked?: ISsifSpo[];
    attested?: ISsifSpo[];
    meta?: ISsifMeta;
}
export interface ISsifSessionRequest {
    meta: ISsifMeta;
    template: Record<string, ISsifSpo>;
}
export interface ISsifSessionResponse {
    attestations: Record<string, ISsifSpo>;
}
