import * as enumerations from './enumerations';
import * as interfaces from './interfaces';
export declare class StringEx {
    static createArray(json: string[]): string[];
}
export declare class RecordEx {
    static createArray(json: Record<string, string>): Record<string, string>;
}
export declare class SsifCommitment implements interfaces.ISsifCommitment {
    static create(json: interfaces.ISsifCommitment): interfaces.ISsifCommitment;
}
export declare class SsifStatements implements interfaces.ISsifStatements {
    static create(json: interfaces.ISsifStatements): interfaces.ISsifStatements;
}
export declare class SsifSpo implements interfaces.ISsifSpo {
    '@context': string[];
    '@type': string;
    statements: interfaces.ISsifStatements;
    commitment?: interfaces.ISsifCommitment;
    static create(json: interfaces.ISsifSpo): interfaces.ISsifSpo;
    static createArray(json: interfaces.ISsifSpo[]): interfaces.ISsifSpo[];
}
export declare class SsifTrustedAttestor implements interfaces.ISsifTrustedAttestor {
    type: enumerations.SsifTrustIssuerType;
    name: string;
    static create(json: interfaces.ISsifTrustedAttestor): interfaces.ISsifTrustedAttestor;
    static createArray(json: interfaces.ISsifTrustedAttestor[]): interfaces.ISsifTrustedAttestor[];
}
export declare class SsifAttest implements interfaces.ISsifAttest {
    iri: string;
    value: string;
    static create(json: interfaces.ISsifAttest): interfaces.ISsifAttest;
}
export declare class SsifAttestationIssuerSpo extends SsifSpo implements interfaces.ISsifSpo {
    static create(json: interfaces.ISsifSpo): interfaces.ISsifSpo;
}
export declare class SsifAttestationSpo extends SsifSpo implements interfaces.ISsifSpo {
    static create(json: interfaces.ISsifSpo): interfaces.ISsifSpo;
}
export declare class SsifText implements interfaces.ISsifText {
    langcode: string;
    text: string;
    static create(json: interfaces.ISsifText): interfaces.ISsifText;
    static createArray(json: interfaces.ISsifText[]): interfaces.ISsifText[];
}
export declare class SsifVerifyRequest implements interfaces.ISsifVerifyRequest {
    iri: string;
    value: string;
    trustedAttestor: interfaces.ISsifTrustedAttestor[];
    static create(json: interfaces.ISsifVerifyRequest): interfaces.ISsifVerifyRequest;
}
export declare class SsifAttestation implements interfaces.ISsifAttestation {
    name: string;
    spo: interfaces.ISsifSpo;
    static create(json: interfaces.ISsifAttestation): interfaces.ISsifAttestation;
    static createArray(json: interfaces.ISsifAttestation[]): interfaces.ISsifAttestation[];
}
export declare class SsifAttestor implements interfaces.ISsifAttestor {
    name: string;
    ethereumAdress: string;
    logo: string;
    infoUrl: string;
    pssifUrl: string;
    texts: interfaces.ISsifText[];
    static create(json: interfaces.ISsifAttestor): interfaces.ISsifAttestor;
    static createArray(json: interfaces.ISsifAttestor[]): interfaces.ISsifAttestor[];
}
export declare class SsifCategory implements interfaces.ISsifCategory {
    name: string;
    texts: interfaces.ISsifText[];
    static create(json: interfaces.ISsifCategory): interfaces.ISsifCategory;
    static createArray(json: interfaces.ISsifCategory[]): interfaces.ISsifCategory[];
}
export declare class SsifMeta implements interfaces.ISsifMeta {
    attestationIssuer: interfaces.ISsifSpo;
    static create(json: interfaces.ISsifMeta): interfaces.ISsifMeta;
}
export declare class SsifPredicate implements interfaces.ISsifPredicate {
    iri: string;
    datatype: string;
    texts: interfaces.ISsifText[];
    categoryName: string;
    attestorNames: string[];
    static create(json: interfaces.ISsifPredicate): interfaces.ISsifPredicate;
    static createArray(json: interfaces.ISsifPredicate[]): interfaces.ISsifPredicate[];
}
export declare class SsifRequest implements interfaces.ISsifRequest {
    attest?: Record<string, string>;
    revoke?: string[];
    verifyRequest?: string[];
    static create(json: interfaces.ISsifRequest): interfaces.ISsifRequest;
}
export declare class SsifAttestorInformation implements interfaces.ISsifAttestorInformation {
    name: string;
    ethereumAdress: string;
    logo: string;
    infoUrl: string;
    ssifUrl: string;
    texts: interfaces.ISsifText[];
    responsePredicates: interfaces.ISsifPredicate[];
    static create(json: interfaces.ISsifAttestorInformation): interfaces.ISsifAttestorInformation;
}
export declare class SsifConfiguration implements interfaces.ISsifConfiguration {
    blockchain?: string;
    privateKey?: string;
    categories?: interfaces.ISsifCategory[];
    sessionTimeout?: number;
    gasMultiplier?: number;
    gasLimit?: number;
    gasPriceMax?: number;
    static create(json: interfaces.ISsifConfiguration): interfaces.ISsifConfiguration;
}
export declare class SsifCreateRequest extends SsifRequest implements interfaces.ISsifCreateRequest {
    publicExtendedKey: string;
    static create(json: interfaces.ISsifCreateRequest): interfaces.ISsifCreateRequest;
}
export declare class SsifCreateRequestAnonymous extends SsifRequest implements interfaces.ISsifRequest {
    static create(json: interfaces.ISsifRequest): interfaces.ISsifRequest;
}
export declare class SsifResult implements interfaces.ISsifResult {
    verified?: interfaces.ISsifSpo[];
    revoked?: interfaces.ISsifSpo[];
    attested?: interfaces.ISsifSpo[];
    meta?: interfaces.ISsifMeta;
    static create(json: interfaces.ISsifResult): interfaces.ISsifResult;
}
export declare class SsifSessionRequest implements interfaces.ISsifSessionRequest {
    meta: interfaces.ISsifMeta;
    template: Record<string, SsifSpo>;
    static create(json: interfaces.ISsifSessionRequest): interfaces.ISsifSessionRequest;
}
export declare class SsifSessionResponse implements interfaces.ISsifSessionResponse {
    attestations: Record<string, SsifSpo>;
    static create(json: interfaces.ISsifSessionResponse): interfaces.ISsifSessionResponse;
}
