"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const NodeCache = require("node-cache");
const uuidv4 = require('uuid/v4');
class Sessions {
    constructor(sessionTimeout = 300) {
        this.checkperiod = 60;
        console.log('sessionTimeout:', sessionTimeout);
        this.cache = new NodeCache({ stdTTL: sessionTimeout, checkperiod: this.checkperiod });
    }
    static generateSessionId() {
        return uuidv4();
    }
    setSsifCreateRequest(key, ssifCreateRequest) {
        this.cache.set(`${key}:create`, ssifCreateRequest);
    }
    setSsifSessionRequest(key, ssifSessionRequest) {
        this.cache.set(`${key}:request`, ssifSessionRequest);
    }
    setSsifSessionResponse(key, ssifSessionResponse) {
        this.cache.set(`${key}:response`, ssifSessionResponse);
    }
    setPublicExtendedKey(key, publicExtendedKey) {
        this.cache.set(`${key}:publicExtendedKey`, publicExtendedKey);
    }
    getSsifCreateRequest(key) {
        return this.cache.get(`${key}:create`);
    }
    getSsifSessionRequest(key) {
        return this.cache.get(`${key}:request`);
    }
    getSsifSessionResponse(key) {
        return this.cache.get(`${key}:response`);
    }
    getPublicExtendedKey(key) {
        return this.cache.get(`${key}:publicExtendedKey`);
    }
    deleteSession(key) {
        this.cache.del([
            `${key}:create`,
            `${key}:request`,
            `${key}:response`,
            `${key}:publicExtendedKey`
        ]);
    }
}
exports.Sessions = Sessions;
exports.default = Sessions;
//# sourceMappingURL=sessions.js.map