'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const erc780_hdkey_hash_2018_1 = require("./erc780-hdkey-hash-2018");
const models_1 = require("./models");
const verifier = require("./verifier");
const sessions_1 = require("./sessions");
const sleep = require("sleep-promise");
class SsifApi {
    constructor(ethereumProvider, contractAddress, privateKey, config = {}) {
        this._jsonLdAttestation = (issuer, publicExtendedKey, predicate, object) => {
            const morphed = {
                '@context': this.context,
                '@type': 'Attestation',
                'statements': {
                    '@id': 'ecdsa-koblitz-pubkey:' +
                        this._attestor.ecdsaKoblitzPubKeyFromPubKey(this._attestor.pubKeyFromPubExtKey(publicExtendedKey, predicate))
                },
                'commitment': {
                    'type': 'SsifErc780Hash2018',
                    'issuer': 'ethereum-address:' + issuer,
                    'publicKey': this._attestor.pubKeyFromPubExtKey(publicExtendedKey, predicate)
                }
            };
            const iriShort = predicate.split('/').pop();
            morphed.statements[iriShort] = object || '';
            return morphed;
        };
        this.addTrustEcdsaKoblitzSignature2016 = (key) => {
            verifier.addTrustedPublicKey(key);
        };
        this.addTrustSSIFEthereumHash2017 = (contract) => {
            verifier.addTrustedContract(contract);
        };
        this.addTrustSsifErc780Hash2018 = (issuer) => {
            verifier.addTrustedIssuer(issuer);
        };
        this.startSessionAutonomous = (ssifCreateRequest) => {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const sessionId = SsifApi.generateSessionId();
                    this._sessions.setSsifCreateRequest(sessionId, ssifCreateRequest);
                    this._sessions.setPublicExtendedKey(sessionId, ssifCreateRequest.publicExtendedKey);
                    const result = yield this.updateAttestations(sessionId, ssifCreateRequest.publicExtendedKey);
                    result.meta = this.meta;
                    resolve(result);
                    return;
                }
                catch (err) {
                    reject(`Error updating attestations: ${err}`);
                }
            }));
        };
        this.startSession = (sessionId, ssifCreateRequest) => {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                // Delete this session - in case that an old one is still present
                this._sessions.deleteSession(sessionId);
                const ssifSessionRequest = models_1.SsifSessionRequest.create({
                    meta: this.meta,
                    template: {}
                });
                // Use 1 attest/revoke key to verify ownership
                const iriAttest = ssifCreateRequest.attest ? Object.keys(ssifCreateRequest.attest) : [];
                const iriRevoke = ssifCreateRequest.revoke ? ssifCreateRequest.revoke : [];
                const sessionIri = iriAttest.concat(iriRevoke)[0];
                if (sessionIri) {
                    ssifSessionRequest.template['session'] = models_1.SsifSpo.create({
                        '@context': this._context,
                        '@type': 'Attestation',
                        statements: {
                            requestedPredicate: sessionIri,
                            session: sessionId
                        },
                        commitment: {
                            type: 'EcdsaKoblitzSignature2016',
                            creator: 'ecdsa-koblitz-pubkey:' + this._attestor.ecdsaKoblitzPubKeyFromPubKey(this._attestor.pubKeyFromPubExtKey(ssifCreateRequest.publicExtendedKey, sessionIri))
                        }
                    });
                }
                if (ssifCreateRequest.verifyRequest && ssifCreateRequest.verifyRequest.length) {
                    ssifCreateRequest.verifyRequest.forEach(requestIri => {
                        const shortIri = requestIri.split('/').pop();
                        ssifSessionRequest.template[shortIri] = models_1.SsifSpo.create({
                            '@context': this._context,
                            '@type': 'Attestation',
                            statements: {},
                            commitment: {
                                'type': { in: ['EcdsaKoblitzSignature2016', 'SSIFEthereumHash2017', 'SsifErc780Hash2018'] }
                            }
                        });
                        ssifSessionRequest.template[shortIri].statements[shortIri] = '?:';
                    });
                }
                this._sessions.setSsifCreateRequest(sessionId, ssifCreateRequest);
                this._sessions.setSsifSessionRequest(sessionId, ssifSessionRequest);
                this._sessions.setPublicExtendedKey(sessionId, ssifCreateRequest.publicExtendedKey);
                while (this._sessions.getSsifSessionRequest(sessionId)) {
                    const ssifSessionResponse = this._sessions.getSsifSessionResponse(sessionId);
                    if (ssifSessionResponse) {
                        const result = {
                            statements: {}
                        };
                        ssifCreateRequest.verifyRequest.forEach(iri => {
                            const shortIri = iri.split('/').pop();
                            try {
                                result.statements[iri] = ssifSessionResponse.attestations[shortIri].statements[shortIri];
                            }
                            catch (_a) {
                                // If it's not there just skip ..
                            }
                            ;
                        });
                        // this._sessions.deleteSession(sessionId);
                        resolve(result);
                        return;
                    }
                    yield sleep(100);
                }
                reject('Timeout');
            }));
        };
        this.startSessionFirstContact = (sessionId, ssifCreateRequest) => {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                // Delete this session - in case that an old one is still present
                this._sessions.deleteSession(sessionId);
                const ssifSessionRequest = models_1.SsifSessionRequest.create({
                    meta: this.meta,
                    template: {}
                });
                ssifSessionRequest.template['HDPublicKey'] = models_1.SsifSpo.create({
                    '@context': this._context,
                    '@type': 'Attestation',
                    statements: {
                        'HDPublicKey': '?:$xpub',
                        'session': sessionId,
                    },
                    commitment: {
                        type: 'EcdsaKoblitzSignature2016',
                        creator: 'ecdsa-koblitz-pubkey:?!address($xpub)'
                    }
                });
                if (ssifCreateRequest.verifyRequest && ssifCreateRequest.verifyRequest.length) {
                    ssifCreateRequest.verifyRequest.forEach(requestIri => {
                        const shortIri = requestIri.split('/').pop();
                        ssifSessionRequest.template[shortIri] = models_1.SsifSpo.create({
                            '@context': this._context,
                            '@type': 'Attestation',
                            statements: {},
                            commitment: {
                                'type': { in: ['EcdsaKoblitzSignature2016', 'SSIFEthereumHash2017', 'SsifErc780Hash2018'] }
                            }
                        });
                        ssifSessionRequest.template[shortIri].statements[shortIri] = '?:';
                    });
                }
                this._sessions.setSsifCreateRequest(sessionId, ssifCreateRequest);
                this._sessions.setSsifSessionRequest(sessionId, ssifSessionRequest);
                while (this._sessions.getSsifSessionRequest(sessionId)) {
                    const ssifSessionResponse = this._sessions.getSsifSessionResponse(sessionId);
                    if (ssifSessionResponse) {
                        const result = {
                            publicExtendedKey: ssifSessionResponse.attestations['HDPublicKey'].statements['HDPublicKey'],
                            statements: {}
                        };
                        ssifCreateRequest.verifyRequest.forEach(iri => {
                            const shortIri = iri.split('/').pop();
                            try {
                                result.statements[iri] = ssifSessionResponse.attestations[shortIri].statements[shortIri];
                            }
                            catch (_a) {
                                // If it's not there just skip ..
                            }
                            ;
                        });
                        // this._sessions.deleteSession(sessionId);
                        resolve(result);
                        return;
                    }
                    yield sleep(100);
                }
                reject('Timeout');
            }));
        };
        this.getSession = (sessionId) => {
            // console.log('getSession parameter getSession:', sessionId);
            return new Promise((resolve, reject) => {
                const result = this._sessions.getSsifSessionRequest(sessionId);
                result ? resolve(result) : reject(`Unknown session ${sessionId}`);
            });
        };
        this.postSession = (sessionId, response) => {
            // console.log('postSession parameter sessionId:', sessionId);
            // console.log('postSession paramater response:', JSON.stringify(response, null, 2));
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                const result = models_1.SsifResult.create({
                    attested: [],
                    revoked: [],
                    verified: [],
                    meta: this.meta
                });
                // Get stored session info (the key + request). We will check response against the original request
                let publicExtendedKey = this._sessions.getPublicExtendedKey(sessionId);
                const request = this._sessions.getSsifSessionRequest(sessionId);
                // No original request found? Reject!
                if (!request) {
                    reject(`Unknown session ${sessionId}`);
                    return;
                }
                // HDPublicKey (FirstContact) FIRST
                if (!publicExtendedKey && request.template['HDPublicKey']) {
                    // If no key is set and a key was requested then we expect it to be present
                    try {
                        publicExtendedKey = response.attestations['HDPublicKey'].statements['HDPublicKey'];
                    }
                    catch (err) {
                        reject('No HDPublicKey statement in response');
                        return;
                    }
                }
                let promises = [];
                // SESSION challenge SECOND
                if (request.template['session']) {
                    try {
                        const composedSession = Object.assign({}, request.template['session']);
                        composedSession['commitment'] = response.attestations['session'].commitment;
                        const ecdsaKoblitzPubKey = response.attestations.session.commitment.creator.replace(/^ecdsa-koblitz-pubkey:/, '');
                        const trustedPublicKeys = {};
                        trustedPublicKeys[ecdsaKoblitzPubKey] = verifier.formatPublicKey(ecdsaKoblitzPubKey);
                        promises.push(verifier.verifyECDSAAttestation(composedSession, { trustedPublicKeys }));
                    }
                    catch (err) {
                        reject(`Session error: ${err}`);
                        return;
                    }
                }
                // VERIFICATIONS challenge THIRD
                let missingAttestations = false;
                for (const req in request.template) {
                    if (req !== 'HDPublicKey' && req !== 'session') {
                        if (response && response.attestations && response.attestations[req]) {
                            console.log('Found in response:', req);
                            promises.push(verifier.verifyAttestation(response.attestations[req]));
                            result.verified.push(response.attestations[req]);
                        }
                        else {
                            console.log(`Missing in response: ${req}`);
                            missingAttestations = true;
                        }
                    }
                }
                // CHECK ALL VERIFICATION PROMISES TO RESOLVE
                try {
                    yield Promise.all(promises);
                    if (missingAttestations) {
                        this._sessions.setSsifSessionResponse(sessionId, response);
                        delete result.attested;
                        delete result.revoked;
                        console.log('Missing attestations...');
                        resolve(result);
                        return;
                    }
                    else {
                        console.log('Verification successful!');
                        // The SsifSessionResponse variable is polled by the startSsifSession call ...
                        // Setting it will cause the startSsifSession to finish and return the result
                        this._sessions.setSsifSessionResponse(sessionId, response);
                    }
                }
                catch (err) {
                    reject(`Verification error: ${err}`);
                    return;
                }
                if (result.verified.length === 0) {
                    delete result.verified;
                }
                // VERIFICATION SUCCESFUL - CONTINUE
                try {
                    const updateResult = yield this.updateAttestations(sessionId, publicExtendedKey);
                    result.attested = updateResult.attested;
                    result.revoked = updateResult.revoked;
                    resolve(result);
                }
                catch (err) {
                    reject(`Error updating attestations: ${err}`);
                }
            }));
        };
        this.updateAttestations = (sessionId, publicExtendedKey) => __awaiter(this, void 0, void 0, function* () {
            const create = this._sessions.getSsifCreateRequest(sessionId);
            const promises = [];
            const result = models_1.SsifResult.create({
                attested: [],
                revoked: []
            });
            // ATTEST
            for (const predicate in create.attest) {
                if (predicate) {
                    console.log(`attesting: ${predicate}: ${create.attest[predicate]}`);
                    promises.push(this._attestor.attestPrivate(publicExtendedKey, predicate, create.attest[predicate]));
                    result.attested.push(this._jsonLdAttestation(this._attestor.address, publicExtendedKey, predicate, create.attest[predicate]));
                }
            }
            // REVOKE
            for (const predicate of create.revoke) {
                if (predicate) {
                    console.log('revoking:', predicate);
                    promises.push(this._attestor.revokePrivate(publicExtendedKey, predicate));
                    result.revoked.push(this._jsonLdAttestation(this._attestor.address, publicExtendedKey, predicate, create.revoke[predicate]));
                }
            }
            return Promise.all(promises).then(() => {
                if (result.attested.length === 0) {
                    delete result.attested;
                }
                if (result.revoked.length === 0) {
                    delete result.revoked;
                }
                return result;
            });
        });
        this._sessions = new sessions_1.Sessions(config.sessionTimeout);
        this._attestor = new erc780_hdkey_hash_2018_1.Erc780HdKeyHash2018(ethereumProvider, contractAddress, privateKey, {
            gasMultiplier: config.gasMultiplier,
            gasLimit: config.gasLimit,
            gasPriceMax: config.gasPriceMax
        });
        // Hook the erc780 verifier function in the verifier module
        verifier.setErc780HdKeyHash2018verifyFunction(this._attestor.verify);
    }
    get context() {
        return this._context;
    }
    set context(ctx) {
        this._context = ctx;
    }
    static generateSessionId() {
        return sessions_1.Sessions.generateSessionId();
    }
    getSchema() {
        const schema = {
            '@context': {
                '@vocab': 'http://ssif/schema/',
                'statements': {
                    '@type': '@id'
                },
                'Attestation': 'Attestation'
            }
        };
        return new Promise((resolve, reject) => {
            schema ? resolve(schema) : reject("Error setting schema");
        });
    }
}
exports.SsifApi = SsifApi;
exports.default = SsifApi;
__export(require("./models"));
//# sourceMappingURL=ssif-api.js.map