"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ssif_api_1 = require("./ssif-api");
const sessionId = ssif_api_1.SsifApi.generateSessionId();
const ethereumProvider = "https://rinkeby.infura.io";
const contractAddress = "0x72d3bc4da6e025540edbfee8b376918f95ec59e4";
const publicExtendedKey = 'xpub69kmhMZrT9bUujrwnJDpE2pyKHeoF8RcqYwoapyZHieEcSQoYtwS7t2KCB16PdwP27UG4xscE6YizKi2VvKggvRNJt5zKzcqZzG5EdfoEVA';
const ssifApi = new ssif_api_1.SsifApi(ethereumProvider, contractAddress);
ssifApi.context = [
    "http://schema.org/",
    "https://w3id.org/security/v1",
    "https://w3id.org/identity/v1",
    "https://uwv.hypoloan.nl/ssif/schema.json"
];
ssifApi.meta = {
    "attestationIssuer": {
        "@context": [
            "http://schema.org/",
            "https://w3id.org/security/v1",
            "https://w3id.org/identity/v1",
            "https://uwv.hypoloan.nl/ssif/schema.json"
        ],
        "@type": "Attestation",
        "statements": {
            "attestationIssuer": "UWV",
            "type": "SSIFEthereumHash2017",
            "contract": "0x960619a3257c88c0bB2dCD56aA2cc822668703C9",
            "logo": "https://www.reprisemedia.com/wp-content/uploads/2017/05/logo-bkr.png"
        }
    }
};
const create = ssif_api_1.SsifCreateRequest.create({
    publicExtendedKey: publicExtendedKey,
    attest: {},
    revoke: [],
    verifyRequest: [
        'http://schema.org/addressCountry',
        'http://schema.org/addressLocality'
    ]
});
(() => __awaiter(this, void 0, void 0, function* () {
    console.log('\n\nsessionId:\n\n', sessionId);
    console.log('\n\ncreate:\n\n', JSON.stringify(create, null, 2));
    // Browser makes request:
    ssifApi.startSession(sessionId, create).then(sessionResponse => {
        console.log('\n\nresponse:\n\n', JSON.stringify(sessionResponse, null, 2));
    });
    // ID-App reads template
    const idAppSessionRequest = ssifApi.getSession(sessionId);
    console.log('\n\nid-app / get:\n\n', JSON.stringify(idAppSessionRequest, null, 2));
    // ID-App posts answer
    const idAppSessionResponse = ssif_api_1.SsifSessionResponse.create({
        attestations: {}
    });
    const idAppResult = yield ssifApi.postSession(sessionId, idAppSessionResponse);
    console.log('\n\nid-app / post:\n\n', JSON.stringify(idAppSessionResponse, null, 2));
    console.log('\n\nid-app / result:\n\n', JSON.stringify(idAppResult, null, 2));
}))();
//# sourceMappingURL=test.js.map